/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: WsmgClient.java,v 1.7 2007/03/21 09:16:04 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.monitor;

import java.io.IOException;
import java.net.URI;
import java.net.URL;

import org.xmlpull.infoset.XmlBuilderException;
import org.xmlpull.infoset.XmlElement;

import wsmg.NotificationHandler;
import wsmg.WseClientAPI;
import wsmg.XmlConsumer;
import wsmg.pull.MessagePuller;
import wsmg.util.WsmgUtil;
import xsul.ws_addressing.WsaEndpointReference;
import xsul5.MLogger;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class WsmgClient implements NotificationHandler {

    private static final MLogger logger = MLogger.getLogger();

    private Monitor monitor;

    private URI brokerURL;

    private String topic;

    private boolean pullMode;

    private URI messageBoxURL;

    private WseClientAPI wseClient;

    private String subscriptionID;

    private MessagePuller messagePuller;

    private XmlConsumer xmlConsumer;

    /**
     * Constructs a WsmgClient.
     * 
     * @param monitor
     */
    public WsmgClient(Monitor monitor) {
        this.monitor = monitor;

        MonitorConfiguration configuration = monitor.getConfiguration();
        // We need to copy these because the configuration might change.
        this.brokerURL = configuration.getBrokerURL();
        this.topic = configuration.getTopic();
        this.pullMode = configuration.isPullMode();
        this.messageBoxURL = configuration.getMessageBoxURL();

        this.wseClient = new WseClientAPI();
    }

    /**
     * Subscribes to the notification.
     * 
     * @throws MonitorException
     */
    public synchronized void subscribe() throws MonitorException {
        if (this.subscriptionID != null) {
            throw new IllegalStateException();
        }
        try {
            if (this.pullMode) {
                WsaEndpointReference messageBoxEPR = this.wseClient
                        .createPullMsgBox(this.messageBoxURL.toString());
                URI address = messageBoxEPR.getAddress();
                this.subscriptionID = this.wseClient.subscribe(this.brokerURL
                        .toString(), address.toString(), this.topic);
                this.messagePuller = this.wseClient
                        .startPullingEventsFromMsgBox(messageBoxEPR, this,
                                1000L);
            } else {
                this.xmlConsumer = new XmlConsumer(0, this);
                this.xmlConsumer.start();
                URL consumerUrl = new URL(this.xmlConsumer.getServer()
                        .getLocation());
                this.subscriptionID = this.wseClient.subscribe(this.brokerURL
                        .toString(), consumerUrl.getHost() + ":"
                        + consumerUrl.getPort(), this.topic);
            }
        } catch (IOException e) {
            throw new MonitorException("Failed to subscribe.", e);
        } catch (RuntimeException e) {
            throw new MonitorException("Failed to subscribe.", e);
        }
    }

    /**
     * Unsubscribes from the notification.
     * 
     * @throws MonitorException
     */
    public synchronized void unsubscribe() throws MonitorException {
        // This method needs to be synchronized along with subscribe() because
        // unsubscribe() might be called while subscribe() is being executed.
        if (this.subscriptionID == null) {
            throw new IllegalStateException();
        }
        try {
            if (this.pullMode) {
                this.messagePuller.stopPulling();
            } else {
                this.xmlConsumer.shutdown();
            }
            this.wseClient.unSubscribe(this.brokerURL.toString(),
                    this.subscriptionID);
        } catch (RuntimeException e) {
            throw new MonitorException("Failed to unsubscribe.", e);
        }

    }

    /**
     * @see wsmg.NotificationHandler#handleNotification(java.lang.String)
     */
    public void handleNotification(String message) {
        try {
            String soapBody = WsmgUtil.getSoapBodyContent(message);
            XmlElement event = XMLUtil.stringToXmlElement(soapBody);
            this.monitor.handleNotification(event);
        } catch (XmlBuilderException e) {
            // Just log them because they can be unrelated messages sent to
            // this topic by accident.
            logger.warning("Could not parse received notification: " + message,
                    e);
        } catch (RuntimeException e) {
            logger.warning("Failed to process notification: " + message, e);
        }
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
